
import warnings

import pandas as pd

from .Extractor import Extractor

warnings.filterwarnings('always')
# warnings.filterwarnings('error')
# warnings.filterwarnings("ignore", category=DeprecationWarning)

class Visualizer(Extractor):
    def __init__(self, ontology: dict, out_dir: str = 'data/visualizer'):
        Extractor.__init__(self, ontology)
        try:
            from pathlib import Path
            self.out_dir = out_dir
            Path(out_dir).mkdir(parents=True, exist_ok=True)
        except OSError as error:
            print("Directory cannot be created")
        self.node_df, self.edge_df = pd.DataFrame(columns=['id']), pd.DataFrame(columns=['from', 'to'])
        self.default_vis_opts = {
            'autoResize': True,
            'height': '1200px',
            'width': '100%',
            'manipulation': 
                {
                    'enabled': True, 
                    'addNode': True
                }
        }

    def deploy_ontology_visualization(self, vis_opts: dict = {}):
        # set vis opts
        if not vis_opts:
            vis_opts = self.default_vis_opts
        # import
        from jaal import Jaal

        # load the data
        self.set_jaal_formatted_ontology()
        # init Jaal and run server
        Jaal(self.edge_df, self.node_df).plot(directed=False, vis_opts=vis_opts)
        
    def set_jaal_formatted_ontology(self):
        self.set_jaal_formatted_ontology_node_df()
        self.set_jaal_formatted_ontology_edge_df()
        
        
    def set_jaal_formatted_ontology_node_df(self):
        id = []
        node_type = []
        data_prop = []
        pi_id = []

        for name in list(self.ontology_df.keys()):
            id += self.ontology_df[name]['main']['label'].tolist() 
            pi_id += self.ontology_df[name]['main']['id'].tolist()
            node_type += len(self.ontology_df[name]['main']['label']) * [name]

        for name in list(self.ontology_df.keys()):
            id += self.ontology_df[name]['data_prop']['data_prop_label'].tolist() 
            pi_id += self.ontology_df[name]['data_prop']['data_prop_id'].tolist()
            node_type += self.ontology_df[name]['data_prop']['data_prop_data_type'].tolist()
        
        node_df = pd.DataFrame(list(zip(id, node_type, pi_id)),
                columns=['id', 'node_type', 'pi_id'])
        node_df['importance'] = node_df['pi_id'].str.count('/').astype(int)
        node_df['importance'] = (node_df['importance'] - node_df['importance'].max() - 1).abs()
        
        # save and delete duplicates
        self.node_df = node_df
        self.node_df[self.node_df.duplicated(subset=['id'], keep=False)].to_excel(f'{self.out_dir}/duplicate_nodes.xlsx')
        self.node_df = self.node_df.drop_duplicates(subset=['id'], keep='first')
        self.node_df.to_excel(f'{self.out_dir}/nodes.xlsx')
        

    
    def set_jaal_formatted_ontology_edge_df(self):
        pi_id_from = []
        pi_id_to = []
        label = []
        edge_type = []
        
        obj_prop_name_mapping = {
            'inheritance_obj_prop': 'Class Inheritance',
            'expansion_obj_prop': 'Object Property Relation',
            'expansion_data_prop': 'Data Property'
        }
        for name in list(self.ontology_df.keys()):      
            for obj_prop_type in ['inheritance_obj_prop', 'expansion_obj_prop', 'expansion_data_prop']:
                pi_id_from += self.ontology_df[name][obj_prop_type]['pi_id_from'].tolist()
                pi_id_to += self.ontology_df[name][obj_prop_type]['pi_id_to'].tolist()
                label += self.ontology_df[name][obj_prop_type]['label'].tolist()
                edge_type += len(self.ontology_df[name][obj_prop_type]['label'].tolist()) * [obj_prop_name_mapping[obj_prop_type]]
            
            
        edge_df = pd.DataFrame(list(zip(pi_id_from, pi_id_to, label, edge_type)),
                columns=['pi_id_from', 'pi_id_to', 'label', 'edge_type'])
    
        edge_df = pd.merge(edge_df, self.node_df[['pi_id', 'id']], left_on='pi_id_from', right_on='pi_id')
        edge_df = pd.merge(edge_df, self.node_df[['pi_id', 'id']], left_on='pi_id_to', right_on='pi_id')
        edge_df = edge_df[['id_x', 'id_y', 'label', 'edge_type']]
        edge_df.columns = ['from', 'to', 'label', 'edge_type']
        
        # faulty ontology
        self.edge_df = edge_df.sort_values('label',ascending=True)
        self.edge_df[self.edge_df.duplicated(subset=['from', 'to'], keep=False)].to_excel(f'{self.out_dir}/duplicate_edges.xlsx')
        self.edge_df = self.edge_df.drop_duplicates(subset=['from', 'to'], keep='first')
        self.edge_df.to_excel(f'{self.out_dir}/edges.xlsx')

        




# import warnings

# import pandas as pd

# from .Extractor import Extractor

# warnings.filterwarnings('always')
# # warnings.filterwarnings('error')
# # warnings.filterwarnings("ignore", category=DeprecationWarning)

# class DBManager(Extractor):
#     def __init__(self, ontology: dict, out_dir: str = 'data/dbmanager'):
#         Extractor.__init__(self, ontology)
#         try:
#             from pathlib import Path
#             Path(out_dir).mkdir(parents=True, exist_ok=True)
#         except OSError as error:
#             print("Directory cannot be created")
    
#     def create_db_schema(self, query_language) -> None:
        
        
#     def create_gsql_db_schema(self) -> None:
#         # https://docs.tigergraph.com/start/gsql-examples/common-applications
#         # https://docs.tigergraph.com/start/gsql-101/define-a-schema
#         # https://towardsdatascience.com/efficient-use-of-tigergraph-and-docker-5e7f9918bf53

#         gsql_ontology_type_mapping = {
#             'TimestampRange': None, #tuple DATETIME DATETIME
#             'Route': None,
#             'Geometric2D': None, 
#             'Coordinate': None, #tuple DOUBLE DOUBLE
#             'ImageLink': 'STRING', 
#             'WebLink': 'STRING', 
#             'Double':'DOUBLE', 
#             'Boolean':'BOOL', 
#             'Text': 'STRING', 
#             'Timestamp':'DATETIME'
#         }
        
# Geometric2D
# Route


        



        



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['piontologyextractor', 'piontologyextractor.Extractor']

package_data = \
{'': ['*']}

install_requires = \
['jaal>=0.0.9,<0.0.10',
 'openpyxl>=3.0.7,<4.0.0',
 'pandas>=1.3.2,<2.0.0',
 'piauthorizer>=0.9.5,<0.10.0']

setup_kwargs = {
    'name': 'piontologyextractor',
    'version': '0.1.1',
    'description': 'A package that can deal with extracting useful info from our Graph Ontology.',
    'long_description': None,
    'author': 'David Berenstien',
    'author_email': 'david.berenstein@pandoraintelligence.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import numpy as np
import pandas as pd


class Extractor:
    def __init__(self, ontology: dict, out_dir: str = 'data/extractor') -> None:
        try:
            from pathlib import Path
            self.out_dir = out_dir
            Path(out_dir).mkdir(parents=True, exist_ok=True)
        except OSError as error:
            print("Directory cannot be created")
        self.ontology = ontology
        self.classes = ontology["classes"]
        self.relationships = ontology["relationships"]
        self.rules = ontology["rules"]
        self.ontology_df = {}
        self.create_ontology_mapping()
        
        
    def create_ontology_mapping(self) -> None:
        self.set_data_classes_from_ontology()
        for name, __loader__ in self.data_class_dict.items():
            self.ontology_df[name] = {}
            self.set_main_ontology(name)
            self.set_expansion_data_properties(name)
            self.set_expansion_object_properties(name)
            self.set_inheritance_object_properties(name)
    
    def set_data_classes_from_ontology(self):
        self.data_class_dict = {}
        self.data_class_dict['esc12'] = [item for item in self.classes if '/esc12' in item['uri']]
        self.data_class_dict['ontology'] = [item for item in self.classes if '/ontology' in item['uri']]
            
    def set_main_ontology(self, name: str):
        ont_id = []
        ont_label = []
        ont_color = []
        ont_inherit_ids = []
        ont_data_prop_ids = []
        ont_obj_prop_ids = []
        data_prop_id = []
        data_prop_data_type = []
        data_prop_label = []
        data_prop_color = []
        data_prop_inherit_ids = []
        obj_prop_id = []
        obj_prop_class_id = []
        label = []
        obj_prop_color = []
        obj_prop_inherit_ids = []

        for ont_item in self.data_class_dict[name]:
            ont_id.append(ont_item['uri'])
            if len(ont_item['annotations']) != 1:
                print(ont_item)
                raise Exception('# annotation != 1')
            ont_data_prop_ids_temp = []
            ont_obj_prop_ids_temp = []
            for prop_item in ont_item['properties']:
                try:
                    if prop_item["$type"] != 'ObjectPropertyInfo':
                        ont_data_prop_ids_temp.append(prop_item['uri'])
                        data_prop_id.append(prop_item['uri'])
                        data_prop_data_type.append(prop_item['dataType'])
                        if len(prop_item['annotations']) != 1:
                            print(prop_item)
                            raise Exception('# annotation != 1')
                        data_prop_label.append(prop_item['annotations'][0]['label'])
                        data_prop_color.append(prop_item['annotations'][0]['color'])
                        data_prop_inherit_ids.append(prop_item['inherits'])
                    else:
                        ont_obj_prop_ids_temp.append(prop_item['uri'])
                        obj_prop_id.append(prop_item['uri'])
                        obj_prop_class_id.append(prop_item['classUri'])
                        if len(prop_item['annotations']) != 1:
                            print(prop_item)
                            raise Exception('# annotation != 1')
                        label.append(prop_item['annotations'][0]['label'])
                        obj_prop_color.append(prop_item['annotations'][0]['color'])
                        obj_prop_inherit_ids.append(prop_item['inherits'])
                except:
                    print(prop_item)
                    raise Exception
            ont_data_prop_ids.append(ont_data_prop_ids_temp)
            ont_obj_prop_ids.append(ont_obj_prop_ids_temp)
            ont_label.append(ont_item['annotations'][0]['label'])
            ont_color.append(ont_item['annotations'][0]['color'])
            ont_inherit_ids.append(ont_item['inherits'])

        df_ont = pd.DataFrame(list(zip(ont_id, ont_label, ont_color, ont_inherit_ids, ont_data_prop_ids, ont_obj_prop_ids)),
                    columns =[f'id', f'label', f'color', f'inherit_id', f'data_prop_id', f'obj_prop_id'])
        if name == 'esc12': # ensure mapping between esc12, entity, ontology
            df_ont.loc[len(df_ont)] = np.array(['http://www.pandoraintelligence.com/esc12', 'esc12', None, ['http://www.pandoraintelligence.com/ontology'], [], []], dtype=object)
        df_ont.to_excel(f'{self.out_dir}/{name}_info.xlsx')
        
        df_data_prop = pd.DataFrame(list(zip(data_prop_id, data_prop_data_type, data_prop_label, data_prop_color, data_prop_inherit_ids)),
                    columns =['data_prop_id', 'data_prop_data_type', 'data_prop_label', 'data_prop_color', 'data_prop_inherit_id'])
        df_data_prop.to_excel(f'{self.out_dir}/{name}_data_prop_info.xlsx')
        
        df_obj_prop = pd.DataFrame(list(zip(obj_prop_id, obj_prop_class_id, label, obj_prop_color, obj_prop_inherit_ids)),
                    columns =['obj_prop_id', 'obj_prop_class_id', 'label', 'obj_prop_color', 'obj_prop_inherit_id'])
        df_obj_prop.to_excel(f'{self.out_dir}/{name}_obj_prop_info.xlsx')
        
        self.ontology_df[name]['main'] = df_ont
        self.ontology_df[name]['data_prop'] = df_data_prop
        self.ontology_df[name]['obj_prop'] = df_obj_prop
    
    def set_expansion_data_properties(self, name):  
        obsolete_columns = ['inherit_id','obj_prop_id','obj_prop_inherit_id', 'data_prop_inherit_id', 'label', 'color', 'data_prop_color']
        df_exploded = self.ontology_df[name]['main'].explode('data_prop_id').reset_index(drop=True)
        df_exploded = df_exploded.explode('obj_prop_id').reset_index(drop=True)
        df_exploded = pd.merge(df_exploded, self.ontology_df[name]['data_prop'], left_on='data_prop_id', right_on='data_prop_id')

        [df_exploded.drop(column, axis=1, inplace=True) for column in obsolete_columns if column in df_exploded.columns]
        df_exploded.drop_duplicates(inplace=True)
        df_exploded = df_exploded.sort_values('id').reset_index(drop=True)
        df_exploded.columns = ['pi_id_from', 'pi_id_to', 'node_type', 'label']
        df_exploded['label'] = '' # formatting
        df_exploded.to_excel(f'{self.out_dir}/{name}_expansion_data_prop.xlsx')
        self.ontology_df[name]['expansion_data_prop'] = df_exploded
    
    def set_expansion_object_properties(self, name):
        df_exploded = self.ontology_df[name]['main'].explode('inherit_id').reset_index(drop=True)
        df_exploded.drop('data_prop_id', axis=1, inplace=True)
        df_exploded.drop('color', axis=1, inplace=True)
        df_exploded.drop('label', axis=1, inplace=True)
        df_exploded.drop('inherit_id', axis=1, inplace=True)
        df_exploded = df_exploded.explode('obj_prop_id').reset_index(drop=True)
        df_exploded = pd.merge(df_exploded, self.ontology_df[name]['obj_prop'], left_on='obj_prop_id', right_on='obj_prop_id')
        df_exploded.drop_duplicates(inplace=True)
        df_exploded = df_exploded[['id', 'obj_prop_id', 'obj_prop_class_id', 'label']]
        df_exploded.columns = ['pi_id_from', 'obj_prop_id', 'pi_id_to', 'label']
        df_exploded.to_excel(f'{self.out_dir}/{name}_expansion_obj_prop.xlsx')
        self.ontology_df[name]['expansion_obj_prop'] = df_exploded
    
    def set_inheritance_object_properties(self, name, inherit_label:str = 'is'):
        df_exploded = self.ontology_df[name]['main'].explode('inherit_id').reset_index(drop=True)
        df_exploded = df_exploded[['inherit_id', 'label', 'id']]
        df_exploded.columns = ['pi_id_from', 'label', 'pi_id_to']
        df_exploded['label'] = f'{inherit_label.capitalize()} ' + df_exploded['label']
        df_exploded['label'] = '' # removed label
        df_exploded['obj_prop_id'] = df_exploded['pi_id_from'] + f'/{"_".join(inherit_label.lower().split())}_' + df_exploded['pi_id_to'].str.split('/').str[-1]

        if name == 'esc12': # ensure mapping between esc12, entity, ontology
            main_ontology_keys = [
                'http://www.pandoraintelligence.com/esc12',
                # 'http://www.pandoraintelligence.com/ontology/entitytype',
                # 'http://www.pandoraintelligence.com/ontology/propertytype'
            ]
            for row in df_exploded.loc[df_exploded.pi_id_from.isnull(), 'pi_id_from'].index:
                df_exploded.at[row, 'pi_id_from'] = main_ontology_keys

            df_exploded = df_exploded.explode('pi_id_from').reset_index(drop=True)
            
            #fix for proper ontology
            for row in df_exploded.loc[df_exploded.obj_prop_id.isnull(), 'obj_prop_id'].index:
                df_exploded.at[row, 'obj_prop_id'] = df_exploded.at[row, 'pi_id_from'] + '/' + str(df_exploded.at[row, 'label']).lower().replace(' ', '_') 
                
        df_exploded.to_excel(f'{self.out_dir}/{name}_inheritance_obj_prop.xlsx')
        self.ontology_df[name]['inheritance_obj_prop'] = df_exploded
